/* EBShape.m created by eb on Mon 14-Dec-1998 */

#import "EBShape.h"

@implementation EBShape

+ (EBShape *) shapeWithType: (EBShapeType) aType
{
  EBShape *aShape = [[[EBShape alloc] initWithType: aType] autorelease];

  return aShape;
}

+ (EBShape *) squareShape
{
  return [self shapeWithType: EBSquare];
}

+ (EBShape *) circleShape
{
  return [self shapeWithType: EBCircle];
}

+ (EBShape *) triangleShape
{
  return [self shapeWithType: EBTriangle];
}

- initWithType: (EBShapeType) aType
{
  [self init];

  [self setShapeType: aType];

  return self;
}

- init
{
  [super init];

  shapeType = EBNoShape;

  return self;
}

- (void) encodeWithCoder: (NSCoder *) coder
{
  [coder encodeValueOfObjCType: @encode(EBShapeType) at: &shapeType];
}

- initWithCoder: (NSCoder *) coder
{
  self = [super init];

  [coder decodeValueOfObjCType: @encode(EBShapeType) at: &shapeType];

  return self;
}

- (int) shapeType
{
  return shapeType;
}

- (void) setShapeType: (EBShapeType) aType
{
  shapeType = aType;
}

- (BOOL) shapeAtCenterPoint: (NSPoint) center
              containsPoint: (NSPoint) aPoint
{
  NSRect aRect = [self shapeFrameWithCenter: center];

  if(shapeType != EBNoShape){
    return NSPointInRect(aPoint,aRect);
  }

  return NO;
}

- (BOOL) shapeAtOriginPoint: (NSPoint) origin
              containsPoint: (NSPoint) aPoint
{
  NSRect aRect = [self shapeFrameWithOrigin: origin];

  if(shapeType != EBNoShape){
    return NSPointInRect(aPoint,aRect);
  }

  return NO;
}

- (NSRect) shapeFrameWithCenter: (NSPoint) center
{
  NSRect aRect = NSMakeRect(center.x - 7.0, center.y - 7.0, 14.0, 14.0);

  return aRect;
}

- (NSRect) shapeFrameWithOrigin: (NSPoint) origin
{
  NSRect aRect = NSMakeRect(origin.x, origin.y, 14.0, 14.0);

  return aRect;
}

@end

@implementation EBShape(Drawing)

- (void) drawShapeAtCenterPoint: (NSPoint) aPoint
{
  NSRect aRect = [self shapeFrameWithCenter: aPoint];

  [self drawShapeInRect: aRect];
}

- (void) drawShapeAtOriginPoint: (NSPoint) aPoint
{
  NSRect aRect = [self shapeFrameWithOrigin: aPoint];

  [self drawShapeInRect: aRect];
}

- (void) drawShapeInRect: (NSRect) aRect
{
  NSRect interiorRect = NSInsetRect(aRect,2.0,2.0);  

  [self drawShapeInInteriorRect: interiorRect];
}

- (void) drawShapeInInteriorRect: (NSRect) aRect
{
  switch(shapeType){
    case EBSquare:
      [self drawSquareInRect: aRect];
      break;
    case EBCircle:
      [self drawCircleInRect: aRect];
      break;
    case EBTriangle:
      [self drawTriangleInRect: aRect];
      break;
    case EBNoShape:
    default:
      [self drawNoShapeInRect: aRect];
      break;
  }
}

- (void) drawNoShapeInRect: (NSRect) aRect
{
  [[NSString stringWithFormat: @"N/A"] drawInRect: aRect
                                   withAttributes: nil];
}

- (void) drawSquareInRect: (NSRect) aRect
{
  [[NSColor blueColor] set];
  NSRectFill(aRect);
}

- (void) drawCircleInRect: (NSRect) aRect
{
  NSPoint center;
  float radius;

  center.x = NSMidX(aRect);
  center.y = NSMidY(aRect);
  radius = (NSHeight(aRect) < NSWidth(aRect)) ?
    (NSHeight(aRect) / 2.0) :
    (NSWidth(aRect) / 2.0);
    
  [[NSColor greenColor] set];
  PSnewpath();
  PSarc(center.x, center.y, radius, 0.0, 360.0);
  PSfill();
}

- (void) drawTriangleInRect: (NSRect) aRect
{

  [[NSColor redColor] set];
  PSnewpath();
  PSmoveto(NSMinX(aRect), NSMinY(aRect));
  PSlineto(NSMaxX(aRect), NSMinY(aRect));
  PSlineto(NSMidX(aRect), NSMaxY(aRect));
  PSclosepath();
  PSfill();
}

- (NSImage *) dragImage
{
  NSRect imageRect = NSMakeRect(0.0, 0.0, 14.0, 14.0);
  NSImage *image = [[NSImage alloc] initWithSize: imageRect.size];

  //[image setFlipped: YES];
  [image lockFocus];
  [[NSColor clearColor] set];
  NSRectFill(imageRect);
  [self drawShapeAtOriginPoint: imageRect.origin];
  [image unlockFocus];

  return image;
}

@end

