/* EBShapeControl.m created by eb on Mon 14-Dec-1998 */

#import "EBShapeControl.h"

@implementation EBShapeControl
- init
{
  [super init];

  location = NSZeroPoint;
  draggingOrigin = NSZeroPoint;
  isDragging = NO;
  // For some reason, this doesn't work here
  // Need to do it after app finishes launching?
  // [self registerForDraggedTypes: [NSArray arrayWithObject: @"EBShape"]];

  return self;
}

- (NSPoint) location
{
  return location;
}

- (void) setLocation: (NSPoint) aLocation
{
  location = aLocation;
  [self setNeedsDisplay: YES];
}

- (NSPoint) draggingOrigin
{
  return draggingOrigin;
}

- (void) setDraggingOrigin: (NSPoint) aPoint
{
  draggingOrigin = aPoint;
  [self setNeedsDisplay: YES];
}

- (BOOL) isDragging
{
  return isDragging;
}

- (void) setIsDragging: (BOOL) flag
{
  isDragging = flag;
  [self setNeedsDisplay: YES];
}

- (BOOL) shapeContainsPoint: (NSPoint) aPoint
{
  return [shape shapeAtOriginPoint: location
                     containsPoint: aPoint];
}

- (NSRect) shapeFrame
{
  return [shape shapeFrameWithOrigin: location];
}

@end

@implementation EBShapeControl(Drawing)
- (void) drawShapeInRect: (NSRect) aRect
{
  BOOL isCurrentlyDragging = [self isDragging];
  NSPoint origin = [self draggingOrigin];

  [shape drawShapeAtOriginPoint: [self location]];
  if(isCurrentlyDragging){
    [self drawPoint: origin];
  }

  if(DelegateHandles(control:didDraw:point:)){
    [delegate control: self
              didDraw: isCurrentlyDragging
                point: origin];
  }

}

- (void) drawPoint: (NSPoint) p
{
  NSString *desc = [NSString stringWithFormat: @"(%.0f,%.0f)",p.x,p.y];
  NSRect bounds = [self bounds];
  NSRect aRect;

  aRect.size = NSMakeSize(60.0, 14.0);
  aRect.origin.y = p.y - 7.0;
  if(p.x - 60.0 < NSMinX(bounds)){
    aRect.origin.x = p.x + 20.0;
  }else{
    aRect.origin.x = p.x - 60.0;
  }

  [desc drawInRect: aRect withAttributes: nil];

} 

@end

@implementation EBShapeControl(EventHandling)
- (void) mouseDownOnShape: (NSEvent *) event
{
  if([event modifierFlags] & NSShiftKeyMask){
    return [super mouseDownOnShape: event];
  }

  [self startInternalDragOnEvent: event];
}
@end

@implementation EBShapeControl(InternalDragging)
- (void) startInternalDragOnEvent: (NSEvent *) event
{
  NSEvent *nextEvent;
  NSPoint p,offset;

  p = [event locationInWindow];
  p = [self convertPoint: p fromView: nil];
  offset.x = p.x - location.x;
  offset.y = p.y - location.y;

  [self setIsDragging: YES];
  [self setDraggingOrigin: [self location]];

  nextEvent = [[self window] nextEventMatchingMask:
    NSLeftMouseUpMask | NSLeftMouseDraggedMask];

  if([nextEvent type] != NSLeftMouseDragged){
    [self setIsDragging: NO];
    return;
  }
 
  do{
    [self updateLocationFromDragEvent: nextEvent
                               offset: offset];
    nextEvent = [[self window] nextEventMatchingMask:
        NSLeftMouseUpMask | NSLeftMouseDraggedMask];
  }while([nextEvent type] != NSLeftMouseUp);

  [self setIsDragging: NO];
  [self updateLocationFromDragEvent: nextEvent
                             offset: offset];
}

- (void) updateLocationFromDragEvent: (NSEvent *) event
                              offset: (NSPoint) offset
{
  NSPoint p,newLocation, newUpperRightCorner;
  NSRect shapeFrame = [self shapeFrame];
  NSRect bounds = [self bounds];

  p = [event locationInWindow];
  p = [self convertPoint: p fromView: nil];

  newLocation.x = p.x - offset.x;
  newLocation.y = p.y - offset.y;
  newUpperRightCorner.x = newLocation.x + NSWidth(shapeFrame);
  newUpperRightCorner.y = newLocation.y + NSHeight(shapeFrame);

  if(NSPointInRect(newLocation,bounds) &&
     NSPointInRect(newUpperRightCorner,bounds)){
    [self setLocation: newLocation];
    [self setDraggingOrigin: newLocation];
  }
}
@end

@implementation EBShapeControl(DraggingDestination)
- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
  NSPasteboard *pb = [sender draggingPasteboard];

  // NSLog(@"Dragging Entered");
  if ([[pb types] indexOfObject:@"EBShape"] != NSNotFound) {
    [self setIsDragging: YES];
    [self draggingUpdated: sender];
    return NSDragOperationAll;
  }

  [self setIsDragging: NO];
  return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
  NSPoint p = [sender draggedImageLocation];

  p = [self convertPoint: p fromView: nil];
  if(!NSEqualPoints(p,[self draggingOrigin])){
    // NSLog(@"Dragging Updated: %@", NSStringFromPoint(p));
    if([self isDragging]){
      [self setDraggingOrigin: p];
    }
  }

  return NSDragOperationAll;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
  // NSLog(@"Dragging Exited");
  [self setIsDragging: NO];
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
  return [self isDragging];
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
  // did my work in conclude drag operation
  // so the app wouldn't block while debugging
  return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
  NSPoint p = [sender draggedImageLocation];
  NSPasteboard *pb = [sender draggingPasteboard];
  NSData *pbData = [pb dataForType: @"EBShape"];
  EBShape *aShape = [NSUnarchiver unarchiveObjectWithData: pbData];

  // NSLog(@"Dragging Concluded");
  [self setShape: aShape];
  p = [self convertPoint: p fromView: nil];
  [self setLocation: p];

  [self setIsDragging: NO];
}

@end

