/* EBShapeVendor.m created by eb on Mon 14-Dec-1998 */

#import "EBShapeVendor.h"

@implementation EBShapeVendor

- init
{
  [super init];

  shape = nil;

  return self;
}
  
- (void) dealloc
{
  [shape release];

  return [super dealloc];
}

- (EBShape *) shape
{
  return shape;
}

- (void) setShape: (EBShape *) aShape
{
  [aShape retain];
  [shape release];
  shape = aShape;
  [self setNeedsDisplay: YES];
}

- delegate
{
  return delegate;
}

- (void) setDelegate: (id) aDelegate
{
  [aDelegate retain];
  [delegate release];
  delegate = aDelegate;
}

- (BOOL) shapeContainsPoint: (NSPoint) aPoint
{
  NSRect bounds = [self bounds];
  NSPoint center = NSMakePoint(NSMidX(bounds), NSMidY(bounds));

  return [shape shapeAtCenterPoint: center
                     containsPoint: aPoint];
}

- (NSRect) shapeFrame
{
  NSRect bounds = [self bounds];
  NSPoint center = NSMakePoint(NSMidX(bounds), NSMidY(bounds));

  return [shape shapeFrameWithCenter: center];
}

@end

@implementation EBShapeVendor(Drawing)
- (void) drawRect: (NSRect) aRect
{
  [self drawBackgroundInRect: aRect];
  [self drawShapeInRect: aRect];
}

- (void) drawBackgroundInRect: (NSRect) aRect
{
  NSRect bounds = [self bounds];
  NSDrawGrayBezel(bounds,bounds);
}

- (void) drawShapeInRect: (NSRect) aRect
{
  NSRect bounds = [self bounds];
  NSPoint center = NSMakePoint(NSMidX(bounds), NSMidY(bounds));

  [shape drawShapeAtCenterPoint: center];
}

@end

@implementation EBShapeVendor(EventHandling)
- (void)mouseDown: (NSEvent *)event
{
  NSPoint p;

  p = [event locationInWindow];
  p = [self convertPoint:p fromView:nil];

  if([self shapeContainsPoint: p]){
    return [self mouseDownOnShape: event];
  }
}

- (void) mouseDownOnShape: (NSEvent *) event
{
  [self startExternalDragOnEvent: event];
}

@end

@implementation EBShapeVendor(DraggingSource)
- (void) startExternalDragOnEvent: (NSEvent *) event
{
  NSPasteboard *pb = [NSPasteboard pasteboardWithName: NSDragPboard];
  NSEvent *nextEvent;
  NSImage *dragImage;
  NSPoint dragPoint,startingPoint;
  NSSize offset;
  NSRect dragFrame;
  
  nextEvent = [[self window] nextEventMatchingMask:
    NSLeftMouseUpMask | NSLeftMouseDraggedMask];

  if([nextEvent type] != NSLeftMouseDragged){
    return;
  }

  startingPoint = [event locationInWindow];
  startingPoint = [self convertPoint: startingPoint fromView: nil];

  dragPoint = [nextEvent locationInWindow];
  dragPoint = [self convertPoint: dragPoint fromView: nil];

  offset.width = dragPoint.x - startingPoint.x;
  offset.height = dragPoint.y - startingPoint.y;

  dragFrame = [self shapeFrame];
  dragImage = [shape dragImage];


  [self declareAndSetShapeOnPasteboard: pb];

  if(DelegateHandles(control:shouldStartExternalDragOnEvent:)){
    if(![delegate control: self shouldStartExternalDragOnEvent: event])
      return;
  }

  [self dragImage: dragImage
               at: dragFrame.origin
           offset: offset
            event: event
       pasteboard: pb
           source: self
        slideBack: YES];

  if(DelegateHandles(control:shouldStartExternalDragOnEvent:)){
    [delegate control: self didFinishExternalDragOnEvent: event];
  }

  [self setNeedsDisplay: YES];
}

- (void) declareAndSetShapeOnPasteboard: (NSPasteboard *) pb
{
  NSData *pbData;

  [pb declareTypes: [NSArray arrayWithObject: @"EBShape"]
             owner: self];

  pbData = [NSArchiver archivedDataWithRootObject: [self shape]];
  [pb setData: pbData forType: @"EBShape"];
}

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)flag
{
  return NSDragOperationAll;
}

@end


